% La fonction "vx" simule et trace le processus (v,x) de la question 4 de
% l'exercice 3.
function vx
% On aura besoin d'utiliser les macros "ALPHA" et "EPSILON" dans la
% fonction "amplitude_saut" ; il faut donc les déclarer comme globales.
global ALPHA EPSILON
VETOILE = 1;
SIGMA2 = 2;
LAMBDA = 1;
% Deux nouveaux paramètres sont à introduire, ainsi que la valeur initiale
% de x.
ALPHA = 2;
EPSILON = .01;
X0 = 0;
V0 = .5;
T = 6;
DT = .002;
nbpas = ceil (T / DT);
temps = (0: nbpas) * DT;
% Maintenant il y a deux vecteurs de valeurs : valeurs_v et valeurs_x.
valeurs_x = zeros (1, nbpas + 1);
valeurs_v = zeros (1, nbpas + 1);
xt = X0;
vt = V0;
valeurs_x(1) = xt;
valeurs_v(1) = vt;
% On lance notre boucle d'Euler stochastique. Cette fois-ci, chaque pas de
% temps va mélanger des phénomènes de saut (pour X_t) et de brownien (pour
% V_t). Pour la simulation de la partie brownienne, nous ne connaissons
% qu'une manière de procéder, donc pas de questionnement là-dessus. Pour la
% simulation des sauts, en revanche, il faut noter qu'ici la densité de
% sauts est très élevée, au point que même pour un pas de temps de .002 il
% y a plusieurs sauts pas pas de temps. On ne peut donc pas faire
% d'approximation qu'il y aurait un saut par pas de temps se produisant
% avec petite probabilité. Ce schéma de simulation deviendrait en revanche
% acceptable pour un DT plus petit (au prix d'un calcul plus lent), voir
% programme "vx_alt".
for i = 1: nbpas
    % On commencer par mettre à jour "xt". Il s'agit d'un processus à
    % sauts, il faut donc prendre des sauts un par un jusqu'à avoir franchi
    % notre pas de temps, par une boucle « while ». "t_loc" désigne le
    % temps écoulé /depuis le début du pas de temps/.
    t_loc = 0;
    while true
        % On utilise une fonction auxiliaire "attente_exp" pour déterminer
        % le temps d'attente avant le prochain saut, voir plus loin.
        t_loc = t_loc + attente_exp (EPSILON ^ (-ALPHA) * vt);
        % Si le prochain saut est censé se produire au-delà de la fin de
        % notre pas de temps, on retient juste l'information qu'il n'y a
        % plus de saut avant la fin du pas de temps et qu'on peut donc
        % sortir de la boucle « while ».
        if t_loc > DT
            break
        end
        % On ajoute à "xt" l'amplitude simulée (par la fonction auxiliaire
        % "amplitude_saut", voir plus loin) du saut qui se produit alors.
        xt = xt + amplitude_saut;
    end
    % Maintenant que nous connaissons la valeur de "xt" à la fin de notre
    % pas de temps, nous pouvons la mémoriser dans "valeurs_x".
    valeurs_x(i + 1) = xt;
    % Mise à jour de "vt" : rien ne change par rapport au code prédécent.
    dW = sqrt (DT) * randn;
    vt = vt + sqrt (SIGMA2) * vt * dW ...
        + (SIGMA2 / 2 - LAMBDA * log (vt / VETOILE)) * vt * DT;
    valeurs_v(i + 1) = vt;
end
% Tracé du résultat. Je mets les deux graphiques dans une même figure,
% grâce à la commande "subplot" que votre camarade Vincent m'a aimablement
% indiquée ; ainsi les axes des abscisses seront automatiquement
% synchronisés (dès lors qu'on utilise les mêmes valeurs d'abscisses à
% plotter sur les deux figures, ce qui est bien le cas ici). (Mais si vous
% préférez faire deux "plot" différents, pas de souci : on n'est pas là
% pour devenir des experts de Matlab ;-) ). La ligne ci-dessous demande à
% MATLAB de créer 2 graphiques superposées en hauteur avec 1 axe des
% abscisses commun, et de sélectionner la sous-figure numéro 1 (celle du
% haut).
subplot (2, 1, 1);
% On trace les valeurs de X. Comme il y a des sauts, j'ai préféré ne pas
% lier les différents point du tracé entre eux, grâce à l'option « '.' ».
plot (temps, valeurs_x, '.');
title ('Évolution de X');
% On sélectionne maintenant la sous-figure numéro 2 (celle du bas).
subplot (2, 1, 2);
% On trace les valeurs de V.
plot (temps, valeurs_v);
title ('Évolution de V')
return
end


% La fonction "attente_exp" simule la réalisation "t" d'une loi
% exponentielle correspondant au temps d'attente avec qu'un processus
% poissonnien de taux "taux" ne « sonne » (ce vocabulaire vient de qu'on
% assimile fréquemment les moments où il se passe quelque chose [en
% l'occurrence, ce sera un saut du processus X] à des sonneries de réveil).
function t = attente_exp (taux)
% Nous avons vu comment simuler les exponentielles lors du dernier TD ; je
% ne reviens fonc pas dessus...
t = -log (rand) / taux;
return
end


% La fonction "amplitude_saut" simule l'amplitude "amplitude" d'un saut
% (conditionnellement à ce qu'il y ait saut), conformément à la solution de
% la question 3.
function amplitude = amplitude_saut
% Notez qu'on va utiliser les valeurs de "ALPHA" et "EPSILON" définies
% (avec une portée globale) dans la fonction-mère, grâce à un coup de
% "global"...
global ALPHA EPSILON
% Simulation de S.
if rand < 1 / 2
    S = -1;
else
    S = 1;
end
% Simulation de U.
U = rand;
% Simulation de "amplitude".
amplitude = EPSILON * S * U ^ (-1 / ALPHA);
return
end
