% Variante de la fonction "vx" où on utilise l'approximation des processus
% de sauts où le pas de temps est très petit devant le temps d'attente
% typique entre deux sauts. Évidemment, on est alors obligé d'augmenter le
% pas de temps, d'où calcul plus lent...
function vx_alt
global ALPHA EPSILON
VETOILE = 1;
SIGMA2 = 2;
LAMBDA = 1;
ALPHA = 2;
EPSILON = .01;
X0 = 0;
V0 = .5;
T = 6;
% Il faut vraiment prendre une valeur de "DT" toute petite...
DT = .000002;
% "DT_PLOT" sera utilisé dans la « ruse démoniaque » vue plus loin.
DT_PLOT = .002;
% Ruse démoniaque : en fait, on ne pas pas tracer /tous/ les pas de temps
% simulés... ! En effet, un "plot" à un million de points est extrêmement
% lourd à tracer pour MATLAB ; donc on va donc sélectionner (au fur et à
% mesure) un sous-ensemble de points à plotter, à peu près un point tous
% les "DT_PLOT". Première conséquence : on ne va plus manipuler un "nbpas"
% correspondant au nombre de pas de simulation, mais un "nbpoints"
% correspondant au nombre de points à tracer... (Il y a un « +1 » car ce
% n'est pas un nombre d'intervalles mais un nombre de points).
nbpoints = ceil (T / DT_PLOT) + 1;
% Dans cette version, à cause de la « ruse démoniaque », on ne sait pas
% encore exactement quels seront les valeurs de temps à plotter (car
% "DT_PLOT" n'est pas nécessairement un multiple de "DT"). On devra donc
% remplir "temps" au fur et à mesure, et on commence donc juste par allouer la
% mémoire pour l'instant...
temps = zeros (1, nbpoints);
valeurs_x = zeros (1, nbpoints);
valeurs_v = zeros (1, nbpoints);
% "t" est le temps courant : à cause de la « ruse démoniaque », on a ici besoin
% de l'enregistrer en mémoire.
t = 0;
xt = X0;
vt = V0;
temps(1) = t;
valeurs_x(1) = xt;
valeurs_v(1) = vt;
% Notre boucle de simulation va comporter une boucle avec (npoints-1)
% passages, à l'intérieur de laquelle il y aura une boucle « while » dans
% laquelle t augmentera par intervalles de temps de DT jusqu'à ce qu'on
% passe le multiple suivant de DT_PLOT.
for i = 1: nbpoints - 1
    % On ne va rien stocker dans la mémoire tant qu'on ne passe pas le
    % multiple suivant de DT_PLOT.
    while t < i * DT_PLOT;
        % Mise à jour de xt, par la méthode de simulation où on dit qu'il y
        % aura en pratique un seul saut au plus, de probabilité
        % proportionnelle à DT.
        if rand < EPSILON ^ (-ALPHA) * vt * DT
            xt = xt + amplitude_saut;
        end
        % Mise à jour de "vt" : rien ne change par rapport au code
        % prédécent.
        dW = sqrt (DT) * randn;
        vt = vt + sqrt (SIGMA2) * vt * dW ...
            + (SIGMA2 / 2 - LAMBDA * log (vt / VETOILE)) * vt * DT;
        % Maintenant il faut aussi mettre à jour "t" lui-même...
        t = t + DT;
    end
    % /Maintenant/, on stocke un triplet de valeurs (t,xt,vt) en mémoire.
    % (Attention au décalage d'indices).
    temps(i + 1) = t;
    valeurs_x(i + 1) = xt;
    valeurs_v(i + 1) = vt;
end
subplot (2, 1, 1);
plot (temps, valeurs_x, '.');
title ('Évolution de X');
subplot (2, 1, 2);
plot (temps, valeurs_v);
title ('Évolution de V')
return
end


function amplitude = amplitude_saut
global ALPHA EPSILON
if rand < 1 / 2
    S = -1;
else
    S = 1;
end
U = rand;
amplitude = EPSILON * S * U ^ (-1 / ALPHA);
return
end
