% Variante de la fonction "vx_alt" où on utilise la technique pour ne pas
% tirer trop de "rand" pendant une attente exponentielle.
function vx_alt_alt
global ALPHA EPSILON
VETOILE = 1;
SIGMA2 = 2;
LAMBDA = 1;
ALPHA = 2;
EPSILON = .01;
X0 = 0;
V0 = .5;
T = 6;
DT = .000002;
DT_PLOT = .002;
nbpoints = ceil (T / DT_PLOT) + 1;
temps = zeros (1, nbpoints);
valeurs_x = zeros (1, nbpoints);
valeurs_v = zeros (1, nbpoints);
t = 0;
xt = X0;
vt = V0;
temps(1) = t;
valeurs_x(1) = xt;
valeurs_v(1) = vt;
% "u" est une variable aléatoire qui suivra toujours une loi uniforme,
% conditionnellement à tout ce qui s'est passé avant. L'astuce est qu'on
% n'utilisera pas forcément la fonction "rand" à chaque fois qu'on aura
% besoin de u, d'où gain de temps.
u = rand;
for i = 1: nbpoints - 1
    while t < i * DT_PLOT;
        % "proba_saut" est la probabilité (infinitésimale) de sauter
        % pendant l'intervalle de temps "DT"
        proba_saut = EPSILON ^ (-ALPHA) * vt * DT;
        % Premier cas : avec probabilité proba_saut, u est très proche de
        % 1. Alors on saute et on retire u.
        if u > 1 - proba_saut
            xt = xt + amplitude_saut;
            u = rand;
            % Si on ne saute pas, on sait que, conditionnellement à cela,
            % (u/(1-proba_saut)) va suivre la loi uniforme sur [0,1].
            % Plutôt que de tirer u, on va donc se contenter de le diviser
            % par (1-proba_saut).
        else
            u = u / (1 - proba_saut);
        end
        dW = sqrt (DT) * randn;
        vt = vt + sqrt (SIGMA2) * vt * dW ...
            + (SIGMA2 / 2 - LAMBDA * log (vt / VETOILE)) * vt * DT;
        t = t + DT;
    end
    temps(i + 1) = t;
    valeurs_x(i + 1) = xt;
    valeurs_v(i + 1) = vt;
end
subplot (2, 1, 1);
plot (temps, valeurs_x, '.');
title ('Évolution de X');
subplot (2, 1, 2);
plot (temps, valeurs_v);
title ('Évolution de V')
return
end


function amplitude = amplitude_saut
global ALPHA EPSILON
if rand < 1 / 2
    S = -1;
else
    S = 1;
end
U = rand;
amplitude = EPSILON * S * U ^ (-1 / ALPHA);
return
end
